/******************************************************************************
 *  Echo Client v1.2.0
 *  Copyright (C) 2002 John David Ratliff and Christopher Michael O'Donnell
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _WIN32
#include <termios.h>
#include <unistd.h>

#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#else
#include <conio.h>
#include <winsock.h>
#endif

#ifdef _WIN32
typedef unsigned long int in_addr_t;
#endif

// default hostname and port
#define	DEFAULT_PORT	64201
#define DEFAULT_HOST	"localhost"

#ifndef _WIN32
// define true/false values
enum TF {FALSE,TRUE};
#endif

// inputs a string from the terminal
void getString(char *buffer, int maxLength) {
	char ch;
	short int pos = 0, done = FALSE;

#ifndef _WIN32
	struct termios normal_mode, input_mode;

	// grab the normal terminal settings
	tcgetattr(STDIN_FILENO, &normal_mode);

	// change the terminal to single-character input mode w/o ECHO
	input_mode = normal_mode;
	input_mode.c_lflag &= ~(ICANON|ECHO);
	input_mode.c_cc[VMIN] = 1;
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &input_mode);
#endif

	// do until we're finished
	while (!done) {
		// get character from standard input
#ifndef _WIN32
		ch = getchar();
#else
		ch = _getch();
#endif

		switch (ch) {
			// did we press ENTER?
			case '\n':
			case '\r':
				buffer[pos] = 0;
				fputc(ch,stdout);
				done = TRUE;
				break;

			// did we press backspace or delete?
			case 0x08:	// Backspace
			case 0x7F:	// Delete
				buffer[--pos] = 0;
				printf("\b \b");
				break;

			// did we press anything else?
			default:
				if (pos < maxLength) {
					buffer[pos++] = ch;
					fputc(ch,stdout);
				}
				break;
		}
	}

#ifndef _WIN32
	// restore original terminal settings
	tcsetattr(STDIN_FILENO, TCSAFLUSH, &normal_mode);
#endif
}

// getIPAddress
// 	returns the network byte ordered IP address for the host named 'hostname'
in_addr_t getIPAddress(char *hostname) {
	struct hostent *h = gethostbyname(hostname);

	if (h == NULL) {
		fprintf(stderr,"Unable to find hostname '%s'\n",hostname);
		return -1;
	}

	// return the IP address in network byte format...
	return inet_addr(inet_ntoa(*((struct in_addr *)h->h_addr)));
}

// opens a connection to the server
// 	returns socket fd on success, -1 on failure
int connectToServer(char *hostname, int port) {
	int sockfd;
	struct sockaddr_in address;

	// open the server socket
	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
		fprintf(stderr,"Unable to open socket!\n");
		return -1;
	}

	// setup address stuffs
	address.sin_family = AF_INET;
	address.sin_addr.s_addr = getIPAddress(hostname);
	address.sin_port = htons((unsigned short)port);

	// connect to the server
	if (connect(sockfd, (struct sockaddr *)&address, sizeof(struct sockaddr)) == -1) {
		fprintf(stderr,"Unable to connect to server!\n");
		return -1;
	}

	return sockfd;
}

int main(int argc, char **argv) {
	char buffer[1024], buf2[1024], *hostname;
	int port = DEFAULT_PORT, sockfd;

#ifdef _WIN32
	WORD wVersionRequested = MAKEWORD(2,2);
	WSADATA wsaData;

	// initialize the Windows Winsockets library
	if (WSAStartup(wVersionRequested,&wsaData) != 0) {
		fprintf(stderr,"Unable to initialize the Winsock library!\n");
		return -1;
	}
#endif

	// erase the message buffers
	memset(buffer,0,1024);
	memset(buf2,0,1024);

	// get the client name
	if (argc > 1) {
		hostname = argv[1];
	} else {
		hostname = DEFAULT_HOST;
	}

	// get the server port, if the user specified one
	if (argc > 2) {
		port = atoi(argv[2]);
	}

	// ask the user for a line to input
	printf("? ");
	getString(buffer,1023);

	// make a connection to the server
	if ((sockfd = connectToServer(hostname,port)) != -1) {
		// send our message to the server
		send(sockfd,buffer,strlen(buffer),0);

		// read the response from the server
		recv(sockfd,buf2,1023,0);

		// print the message back to the standard output
		printf("\nResponse from server: \"%s\"\n",buf2);
	}

#ifdef _WIN32
	// perform Winsock cleanup
	WSACleanup();
#endif

	return EXIT_SUCCESS;
}
